@@ -3,14 +3,14 @@
     mistune
     ~~~~~~~
 
-    Yet another markdown parser, inspired by marked in JavaScript.
+    The fastest markdown parser in pure Python, inspired by marked.
 
     :copyright: (c) 2014 by Hsiaoming Yang.
 """
 
 import re
 
-__version__ = '0.1.0'
+__version__ = '0.2.0'
 __author__ = 'Hsiaoming Yang <me@lepture.com>'
 __all__ = [
     'BlockGrammar', 'BlockLexer',
@@ -174,12 +174,12 @@ class BlockLexer(object):
         src = src.rstrip('\n')
 
         if not features:
-            features = [
+            features = (
                 'newline', 'block_code', 'fences', 'heading',
                 'nptable', 'lheading', 'hrule', 'block_quote',
                 'list_block', 'block_html', 'def_links',
                 'def_footnotes', 'table', 'paragraph', 'text'
-            ]
+            )
 
         def manipulate(src):
             for key in features:
@@ -251,10 +251,10 @@ class BlockLexer(object):
         self.tokens.append({'type': 'list_end'})
 
     def _process_list_item(self, cap, bull):
-        features = [
+        features = (
             'newline', 'block_code', 'fences', 'lheading', 'hrule',
             'block_quote', 'list_block', 'block_html', 'text',
-        ]
+        )
 
         cap = self.rules.list_item.findall(cap)
 
@@ -335,11 +335,11 @@ class BlockLexer(object):
                 newlines.append(line[whitespace:])
             text = '\n'.join(newlines)
 
-        features = [
+        features = (
             'newline', 'block_code', 'fences', 'heading',
             'nptable', 'lheading', 'hrule', 'block_quote',
             'list_block', 'block_html', 'table', 'paragraph', 'text'
-        ]
+        )
 
         self.parse(text, features)
 
@@ -507,13 +507,16 @@ class InlineLexer(object):
                 m = pattern.match(src)
                 if not m:
                     continue
+                self.line_match = m
                 out = getattr(self, 'output_%s' % key)(m)
                 if out is not None:
                     return m, out
             return False
 
+        self.line_started = False
         while src:
             ret = manipulate(src)
+            self.line_started = True
             if ret is not False:
                 m, out = ret
                 output += out
@@ -634,6 +637,7 @@ class Renderer(object):
         :param code: text content of the code block.
         :param lang: language of the given code.
         """
+        code = code.rstrip()
         if not lang:
             code = escape(code, smart_amp=False)
             return '<pre><code>%s\n</code></pre>\n' % code
@@ -836,7 +840,7 @@ class Renderer(object):
             text = re.sub(r'<\/p>$', r'%s</p>' % back, text)
         else:
             text = '%s<p>%s</p>' % (text, back)
-        html = '<li id="fn-%s">%s''</li>\n' % (escape(key), text)
+        html = '<li id="fn-%s">%s</li>\n' % (escape(key), text)
         return html
 
     def footnotes(self, text):
